package com.sportradar.uf.odds;

import java.io.IOException;
import java.math.BigDecimal;
import java.math.MathContext;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class OddsUSConverterUtils {

  private static final Logger log = LoggerFactory.getLogger(OddsUSConverterUtils.class);
  private static List<Integer> ladder = new ArrayList<>();

  static {
    loadUsOddsLadder();
  }

  private OddsUSConverterUtils() {}

  @SuppressWarnings("unchecked")
  private static void loadUsOddsLadder() {
    try {
      IOUtils.readLines(
              OddsUSConverterTest.class.getResourceAsStream("/US Ladder - v2.csv"),
              StandardCharsets.UTF_8.name())
          .stream()
          .mapToInt(l -> Integer.parseInt((String) l))
          .forEach(ladder::add);
    } catch (IOException e) {
      log.error("Error reading ladder file {}", e.getMessage());
    }
  }

  public static int convertAndRoundEuToUsOdds(Double oddsEu) {
    Optional<Double> usOdds = convertEuOddsToUs(oddsEu);
    if (usOdds.isPresent()) {
      return roundUSOdds(usOdds.get(), ladder);
    }
    log.warn("Cannot correctly convert EU odds: {}", oddsEu);
    return -1;
  }

  /**
   * Convert decimal EU odds to decimal US odds
   *
   * <p>if EU_odds >= 2 then US_odds=(EU_odds - 1)
   *
   * <p>100 if EU_odds <2 then US_odds=(-100)/(EU_odds-1)
   *
   * @param oddsEUDouble EU odds not null
   */
  public static Optional<Double> convertEuOddsToUs(Double oddsEUDouble) {
    BigDecimal oddsEu = BigDecimal.valueOf(oddsEUDouble);
    Optional<Double> oddUs;
    if (oddsEu.doubleValue() == 1) {
      oddUs = Optional.empty();
    } else if (oddsEu.doubleValue() >= 2) {
      oddUs =
          Optional.of(
              oddsEu
                  .subtract(BigDecimal.valueOf(1d))
                  .multiply(BigDecimal.valueOf(100))
                  .doubleValue());
    } else {
      oddUs =
          Optional.of(
              (BigDecimal.valueOf(-100d))
                  .divide(oddsEu.subtract(BigDecimal.valueOf(1)), MathContext.DECIMAL128)
                  .doubleValue());
    }

    return oddUs;
  }

  /**
   * Round oddsUs to closest value from ladder.
   *
   * @param oddsUs decimal US odds value not null
   * @param ladder list of US odds values not null
   */
  public static int roundUSOdds(Double oddsUs, List<Integer> ladder) {
    Integer min = Integer.MIN_VALUE;
    Integer max = Integer.MAX_VALUE;
    ladder.sort(Integer::compareTo);
    for (Integer l : ladder) {
      if (oddsUs >= l) {
        min = l;
      }

      if (oddsUs <= l && l < max) {
        max = l;
      }
    }
    return Math.abs(oddsUs - min) <= Math.abs(max - oddsUs) ? min : max;
  }
}
